@@ -41,14 +41,15 @@ struct _XstNetworkInterfaceDescription {
 };
 
 static const XstNetworkInterfaceDescription xst_iface_desc [] = {
-	{ N_("Other type"),                   XST_CONNECTION_OTHER,   "network.png",  "other_type"},
-	{ N_("Ethernet LAN card"),            XST_CONNECTION_ETH,     "16_ethernet.xpm", "eth"   },
-	{ N_("WaveLAN wireless LAN"),         XST_CONNECTION_WVLAN,   "network.png",  "wvlan" },
-	{ N_("PPP: modem or transfer cable"), XST_CONNECTION_PPP,     "16_ppp.xpm",      "ppp"   },
-	{ N_("Parallel line"),                XST_CONNECTION_PLIP,    "16_plip.xpm",  "plip"  },
-	{ N_("Loopback: virtual interface"),  XST_CONNECTION_LO,      "16_loopback.xpm", "lo"    },
-	{ N_("Unknown type"),                 XST_CONNECTION_UNKNOWN, "network.png",  NULL},
-	{ NULL, XST_CONNECTION_UNKNOWN, NULL, NULL}
+	{ N_("Other type"),                   XST_CONNECTION_OTHER,   "network.png",     "other_type" },
+	{ N_("Ethernet LAN card"),            XST_CONNECTION_ETH,     "16_ethernet.xpm", "eth"        },
+	{ N_("WaveLAN wireless LAN"),         XST_CONNECTION_WVLAN,   "wavelan-16.png",  "wvlan"      },
+	{ N_("PPP: modem or transfer cable"), XST_CONNECTION_PPP,     "16_ppp.xpm",      "ppp"        },
+	{ N_("Parallel line"),                XST_CONNECTION_PLIP,    "16_plip.xpm",     "plip"       },
+	{ N_("Infrared LAN"),                 XST_CONNECTION_IRLAN,   "irda-16.png",     "irlan"      },
+	{ N_("Loopback: virtual interface"),  XST_CONNECTION_LO,      "16_loopback.xpm", "lo"         },
+	{ N_("Unknown type"),                 XST_CONNECTION_UNKNOWN, "network.png",     NULL         },
+	{ NULL,                               XST_CONNECTION_UNKNOWN, NULL,              NULL         }
 };
 
 
@@ -61,6 +62,7 @@ static gint on_connection_config_dialog_delete_event (GtkWidget *w, GdkEvent *ev
 static void on_connection_modified (GtkWidget *w, XstConnection *cxn);
 static void on_wvlan_adhoc_toggled (GtkWidget *w, XstConnection *cxn);
 static void on_ppp_update_dns_toggled (GtkWidget *w, XstConnection *cxn);
+static gboolean on_ip_address_focus_out (GtkWidget *widget, GdkEventFocus *event, XstConnection *cxn);
 
 #define W(s) my_get_widget (cxn->xml, (s))
 
@@ -207,8 +209,7 @@ connection_xml_wvsection_search (xmlNode *node, gchar *section_name, gchar *type
 	g_return_val_if_fail (section_name != NULL, NULL);
 	
 	for (node = xst_xml_element_find_first (node, "dialing");
-		node; node = xst_xml_element_find_next (node, "dialing"))
-	{
+	     node; node = xst_xml_element_find_next (node, "dialing")) {
 		if (type && !connection_xml_wvsection_is_type (node, type))
 			continue;
 		
@@ -349,35 +350,6 @@ connection_wvsection_name_generate (gchar *dev, xmlNode *root)
 	return str;
 }
 
-/* dev_type is "eth", "wvlan", "ppp", "plip"... */
-static gchar *
-connection_dev_get_next (xmlNode *root, gchar *dev_type)
-{
-	xmlNode *node;
-	gchar *dev;
-	gint len, max, num;
-
-	len = strlen (dev_type);
-	max = 0;
-
-	for (node = xst_xml_element_find_first (root, "interface");
-		node; node = xst_xml_element_find_next (node, "interface"))
-	{
-		dev = xst_xml_get_child_content (node, "dev");
-
-		g_return_val_if_fail (dev != NULL, NULL);
-		g_return_val_if_fail (dev_type != NULL, NULL);
-		
-		if (strstr (dev, dev_type)) {
-			num = atoi (dev + len) + 1;
-			max = (num > max)? num: max;
-		}
-		g_free (dev);
-	}
-
-	return g_strdup_printf ("%s%d", dev_type, max);
-}
-
 static IPConfigType
 connection_config_type_from_str (gchar *str)
 {
@@ -498,12 +470,335 @@ connection_set_row_pixtext (GtkWidget *clist, gint row, gchar *text, gboolean en
 			       active_mask[enabled ? 1 : 0]);
 }
 
+/* NULL if false, else GtkWidget in data in found node */
+static GtkWidget *
+connection_default_gw_find_item (GtkWidget *omenu, gchar *dev)
+{
+	GList *l;
+	gchar *value;
+
+	for (l = gtk_object_get_data (GTK_OBJECT (omenu), "list");
+	     l; l = l->next) {
+		value = gtk_object_get_data (GTK_OBJECT (l->data), "value");
+		if (!strcmp (dev, value))
+			return l->data;
+	}
+
+	return NULL;
+}
+
+static void
+connection_default_gw_activate (GtkMenuItem *item, gpointer data)
+{
+	gtk_object_set_data (GTK_OBJECT (tool), "gatewaydev", data);
+}
+
+void
+connection_default_gw_add (gchar *dev)
+{
+	GtkWidget *omenu, *menu, *item;
+	GList *l;
+	gchar *cpy;
+
+	if (!strcmp (dev, "lo"))
+		return;
+	
+	omenu = xst_dialog_get_widget (tool->main_dialog, "connection_def_gw_omenu");
+	menu  = gtk_option_menu_get_menu (GTK_OPTION_MENU (omenu));
+
+	if (connection_default_gw_find_item (omenu, dev))
+		return;
+	
+	cpy = g_strdup (dev);
+	item = gtk_menu_item_new_with_label (dev);
+	gtk_signal_connect (GTK_OBJECT (item), "activate",
+			    GTK_SIGNAL_FUNC (connection_default_gw_activate),
+			    (gpointer) cpy);
+	gtk_object_set_data (GTK_OBJECT (item), "value", cpy);
+	gtk_widget_show (item);
+	gtk_menu_append (GTK_MENU (menu), item);
+	
+	l = gtk_object_get_data (GTK_OBJECT (omenu), "list");
+	l = g_list_append (l, item);
+	gtk_object_set_data (GTK_OBJECT (omenu), "list", l);
+}
+
+void
+connection_default_gw_remove (gchar *dev)
+{
+	GtkWidget *omenu, *menu, *item;
+	GList *l;
+	gchar *cpy;
+	
+	omenu = xst_dialog_get_widget (tool->main_dialog, "connection_def_gw_omenu");
+	menu  = gtk_option_menu_get_menu (GTK_OPTION_MENU (omenu));
+
+	g_return_if_fail ((item = connection_default_gw_find_item (omenu, dev)));
+	
+	l = gtk_object_get_data (GTK_OBJECT (omenu), "list");
+	l = g_list_remove (l, item);
+	gtk_object_set_data (GTK_OBJECT (omenu), "list", l);
+
+	cpy = gtk_object_get_data (GTK_OBJECT (item), "value");
+	g_free (cpy);
+	
+	gtk_widget_destroy (item);
+}
+
+void
+connection_default_gw_init (XstTool *tool, gchar *dev)
+{
+	GtkWidget *omenu, *menu, *item;
+
+	omenu = xst_dialog_get_widget (tool->main_dialog, "connection_def_gw_omenu");
+	menu  = gtk_option_menu_get_menu (GTK_OPTION_MENU (omenu));
+
+	item = gtk_menu_get_active (GTK_MENU (menu));
+	gtk_signal_connect (GTK_OBJECT (item), "activate",
+			    GTK_SIGNAL_FUNC (connection_default_gw_activate),
+			    (gpointer) NULL);
+	/* Strange bug caused the "Auto" item to be unsensitive the first time. */
+	gtk_menu_shell_select_item (GTK_MENU_SHELL (menu), item);
+	
+	item = connection_default_gw_find_item (omenu, dev);
+	if (item) {
+		GList *l;
+
+		l = gtk_object_get_data (GTK_OBJECT (omenu), "list");
+		
+		gtk_option_menu_set_history (GTK_OPTION_MENU (omenu), g_list_index (l, item) + 1);
+		gtk_menu_shell_select_item (GTK_MENU_SHELL (menu), item);
+	}
+}
+
+XstConnection *
+connection_default_gw_get_connection (XstTool *tool)
+{
+	gchar *dev;
+
+	dev = gtk_object_get_data (GTK_OBJECT (tool), "gatewaydev");
+	g_return_val_if_fail (dev != NULL, NULL);
+
+	return connection_find_by_dev (tool, dev);
+}
+
+XstConnectionErrorType
+connection_default_gw_check_manual (XstConnection *cxn, gboolean ignore_enabled)
+{
+	g_return_val_if_fail (cxn != NULL, XST_CONNECTION_ERROR_OTHER);
+
+	if (!ignore_enabled && !cxn->enabled)
+		return XST_CONNECTION_ERROR_ENABLED;
+
+	switch (cxn->type) {
+	case XST_CONNECTION_ETH:
+	case XST_CONNECTION_WVLAN:
+	case XST_CONNECTION_IRLAN:
+		if (cxn->ip_config == IP_MANUAL && (!cxn->gateway || !*cxn->gateway))
+			return XST_CONNECTION_ERROR_STATIC;
+		break;
+	case XST_CONNECTION_PPP:
+		if (!cxn->set_default_gw)
+			return XST_CONNECTION_ERROR_PPP;
+		break;
+	case XST_CONNECTION_PLIP:
+		if (!cxn->remote_address || !*cxn->remote_address)
+			return XST_CONNECTION_ERROR_STATIC;
+		break;
+	case XST_CONNECTION_LO:
+	default:
+		g_warning ("connection_default_gw_check_manual: shouldn't be here.");
+		return XST_CONNECTION_ERROR_OTHER;
+	}
+
+	return XST_CONNECTION_ERROR_NONE;
+}
+
+void
+connection_default_gw_fix (XstConnection *cxn, XstConnectionErrorType error)
+{
+	switch (error) {
+	case XST_CONNECTION_ERROR_ENABLED:
+		cxn->enabled = TRUE;
+		break;
+	case XST_CONNECTION_ERROR_PPP:
+		cxn->set_default_gw = TRUE;
+		break;
+	case XST_CONNECTION_ERROR_STATIC:
+	case XST_CONNECTION_ERROR_NONE:
+	case XST_CONNECTION_ERROR_OTHER:
+	default:
+		g_warning ("connection_default_gw_fix: shouldn't be here.");
+	}
+}
+
+void
+connection_default_gw_set_manual (XstTool *tool, XstConnection *cxn)
+{
+	gchar *gateway;
+
+	gateway = gtk_object_get_data (GTK_OBJECT (tool), "gateway");
+	if (gateway)
+		g_free (gateway);
+	gateway = NULL;
+
+	if (!cxn) {
+		gtk_object_set_data (GTK_OBJECT (tool), "gateway", NULL);
+		gtk_object_set_data (GTK_OBJECT (tool), "gatewaydev", NULL);
+		return;
+	}
+
+	switch (cxn->type) {
+	case XST_CONNECTION_ETH:
+	case XST_CONNECTION_WVLAN:
+	case XST_CONNECTION_IRLAN:
+		if (cxn->ip_config == IP_MANUAL)
+			gateway = g_strdup (cxn->gateway);
+		break;
+	case XST_CONNECTION_PLIP:
+		gateway = strdup (cxn->remote_address);
+		break;
+	case XST_CONNECTION_PPP:
+		gtk_object_set_data (GTK_OBJECT (tool), "gatewaydev", NULL);
+		break;
+	case XST_CONNECTION_LO:
+	default:
+		gtk_object_set_data (GTK_OBJECT (tool), "gatewaydev", NULL);
+		g_warning ("connection_default_gw_set_manual: shouldn't be here.");
+		break;
+	}
+
+	gtk_object_set_data (GTK_OBJECT (tool), "gateway", gateway);
+}
+
+static gboolean
+connection_type_is_lan (XstConnectionType type)
+{
+	return (type == XST_CONNECTION_ETH ||
+		type == XST_CONNECTION_WVLAN ||
+		type == XST_CONNECTION_IRLAN);
+
+}
+
+static XstConnection *
+connection_default_gw_find_static (XstTool *tool)
+{
+	XstConnection *cxn;
+	GtkWidget *clist;
+	int i;
+	
+	clist = xst_dialog_get_widget (tool->main_dialog, "connection_list");
+	for (i=0; i < GTK_CLIST (clist)->rows; i++) {
+		cxn = gtk_clist_get_row_data (GTK_CLIST (clist), i);
+
+		/* Try fo find an active ethernet or wavelan connection with a static gateway definded */
+		if (cxn->enabled && connection_type_is_lan (cxn->type) &&
+		    (cxn->ip_config == IP_MANUAL) &&
+		    (cxn->gateway && *cxn->gateway))
+			return cxn;
+	}
+
+	return NULL;
+}
+
+static XstConnection *
+connection_default_gw_find_ppp (XstTool *tool)
+{
+	XstConnection *cxn;
+	GtkWidget *clist;
+	int i;
+	
+	clist = xst_dialog_get_widget (tool->main_dialog, "connection_list");
+	for (i=0; i < GTK_CLIST (clist)->rows; i++) {
+		cxn = gtk_clist_get_row_data (GTK_CLIST (clist), i);
+
+		/* Try fo find an active PPP connection with the default_gw bit on. */
+		if (cxn->enabled &&
+		    cxn->type == XST_CONNECTION_PPP &&
+		    cxn->set_default_gw)
+			return cxn;
+	}
+	
+	return NULL;
+}
+
+static XstConnection *
+connection_default_gw_find_dynamic (XstTool *tool)
+{
+	XstConnection *cxn;
+	GtkWidget *clist;
+	int i;
+	
+	clist = xst_dialog_get_widget (tool->main_dialog, "connection_list");
+	for (i=0; i < GTK_CLIST (clist)->rows; i++) {
+		cxn = gtk_clist_get_row_data (GTK_CLIST (clist), i);
+		/* Try fo find an active ethernet or wavelan connection with dynamic configuration */
+		if (cxn->enabled &&
+		    (cxn->type == XST_CONNECTION_ETH || cxn->type == XST_CONNECTION_WVLAN) &&
+		    (cxn->ip_config != IP_MANUAL))
+			return cxn;
+	}
+	
+	return NULL;
+}
+
+static XstConnection *
+connection_default_gw_find_plip (XstTool *tool)
+{
+	XstConnection *cxn;
+	GtkWidget *clist;
+	int i;
+	
+	clist = xst_dialog_get_widget (tool->main_dialog, "connection_list");
+	for (i=0; i < GTK_CLIST (clist)->rows; i++) {
+		cxn = gtk_clist_get_row_data (GTK_CLIST (clist), i);
+
+		/* Try fo find an active plip connection with a set remote address. */
+		if (cxn->enabled &&
+		    (cxn->type == XST_CONNECTION_PLIP) &&
+		    (cxn->remote_address && *cxn->remote_address))
+			return cxn;
+	}
+
+	return NULL;
+}
+
+void
+connection_default_gw_set_auto (XstTool *tool)
+{
+	XstConnection *cxn;
+
+	if (!(cxn = connection_default_gw_find_static (tool))  &&
+	    !(cxn = connection_default_gw_find_ppp (tool))     &&
+	    !(cxn = connection_default_gw_find_dynamic (tool)) &&
+	    !(cxn = connection_default_gw_find_plip (tool)))
+		cxn = NULL;
+
+	connection_default_gw_set_manual (tool, cxn);
+}
+
+void
+connection_update_clist_enabled_apply (GtkWidget *clist)
+{
+	gint i;
+	XstConnection *cxn;
+
+	g_return_if_fail (GTK_IS_CLIST (clist));
+
+	for (i = 0; i < GTK_CLIST (clist)->rows; i++) {
+		cxn = gtk_clist_get_row_data (GTK_CLIST (clist), i);
+		g_return_if_fail (cxn != NULL);
+		connection_set_row_pixtext (clist, i, cxn->enabled ? _("Active") :
+					    _("Inactive"), cxn->enabled);
+	}
+}
+
 void
 connection_update_row_enabled (XstConnection *cxn, gboolean enabled)
 {
+	XstConnection *cxn2;
 	GtkWidget *clist;
 	gint row, i;
-	XstConnection *cxn2;
 
 	g_return_if_fail (cxn != NULL);
 	g_return_if_fail (cxn->dev != NULL);
@@ -517,16 +812,16 @@ connection_update_row_enabled (XstConnection *cxn, gboolean enabled)
 			g_return_if_fail (cxn2 != NULL);
 			g_return_if_fail (cxn2->dev != NULL);
 			
-			if (!strcmp (cxn->dev, cxn2->dev))
+			if (!strcmp (cxn2->dev, cxn->dev))
 				connection_update_row_enabled (cxn2, FALSE);
 		}
 
 	cxn->enabled = enabled;
 	row = gtk_clist_find_row_from_data (GTK_CLIST (clist), cxn);
 	g_return_if_fail (row > -1);
-	connection_set_row_pixtext (clist, row, enabled ? _("To activate") :
-				    _("To inactivate"), enabled);
-	xst_dialog_modify (tool->main_dialog);
+	connection_set_row_pixtext (clist, row, enabled ? _("Active") :
+				    _("Inactive"), enabled);
+	/*xst_dialog_modify (tool->main_dialog);*/
 }
 
 void
@@ -548,7 +843,7 @@ connection_update_row (XstConnection *cxn)
 	gtk_clist_set_pixtext (GTK_CLIST (clist), row, 0, cxn->dev, GNOME_PAD_SMALL,
 			       mini_pm[cxn->type], mini_mask[cxn->type]);
 
-	gtk_clist_set_text (GTK_CLIST (clist), row, 2, cxn->name);
+	gtk_clist_set_text (GTK_CLIST (clist), row, 2, cxn->name ? cxn->name : "");
 	gtk_clist_sort (GTK_CLIST (clist));
 
 	xst_dialog_modify (tool->main_dialog);
@@ -566,12 +861,31 @@ connection_add_to_list (XstConnection *cxn, GtkWidget *clist)
 	row = gtk_clist_append (GTK_CLIST (clist), text);
 	gtk_clist_set_row_data (GTK_CLIST (clist), row, cxn);
 
+	connection_default_gw_add (cxn->dev);
 	connection_update_row (cxn);
 	connection_update_row_enabled (cxn, cxn->enabled);
 
 	return;
 }
 
+XstConnection *
+connection_find_by_dev (XstTool *tool, gchar *dev)
+{
+	XstConnection *cxn;
+	GtkWidget *clist;
+	int i;
+	
+	clist = xst_dialog_get_widget (tool->main_dialog, "connection_list");
+	for (i=0; i < GTK_CLIST (clist)->rows; i++) {
+		cxn = gtk_clist_get_row_data (GTK_CLIST (clist), i);
+		if (!strcmp (cxn->dev, dev))
+			return cxn;
+	}
+
+	return NULL;
+}
+
+
 /*static void
 add_connections_to_list (void)
 {
@@ -630,7 +944,49 @@ connection_get_ptp_from_node (xmlNode *node, XstConnection *cxn)
 {
 	cxn->remote_address = xst_xml_get_child_content (node, "remote_address");
 }
-	
+
+static gchar *
+connection_find_new_device (xmlNode *root, XstConnectionType type)
+{
+	xmlNode *node;
+	gchar *prefix, *dev, *numstr;
+	gint max;
+	gchar *devs[] = {
+		NULL,
+		"eth",
+		"wvlan",
+		"ppp",
+		"plip",
+		"irlan",
+		"lo",
+		NULL
+	};
+
+	prefix = devs[(gint) type];
+	if (!prefix)
+		return g_strdup ("NIL");
+
+	max = -1;
+
+	for (node = xst_xml_element_find_first (root, "interface");
+	     node; node = xst_xml_element_find_next (node, "interface")) {
+		dev = xst_xml_get_child_content (node, "dev");
+		if (dev) {
+			if (strstr (dev, prefix) == dev) {
+				numstr = dev + strlen (prefix);
+				if (*numstr && max < atoi (numstr))
+					max = atoi (numstr);
+			}
+
+			g_free (dev);
+		}
+	}
+
+	max++;
+
+	return g_strdup_printf ("%s%u", prefix, max);
+}					
+
 XstConnection *
 connection_new_from_type (XstConnectionType type, xmlNode *root)
 {
@@ -638,6 +994,7 @@ connection_new_from_type (XstConnectionType type, xmlNode *root)
 
 	cxn = g_new0 (XstConnection, 1);
 	cxn->type = type;
+	cxn->file = NULL;
 	
 	/* set up some defaults */
 	cxn->user = FALSE;
@@ -645,28 +1002,21 @@ connection_new_from_type (XstConnectionType type, xmlNode *root)
 	cxn->update_dns = TRUE;
 	cxn->ip_config = cxn->tmp_ip_config = IP_MANUAL;
 	
-#warning FIXME: figure out a new device correctly
+	cxn->dev = connection_find_new_device (root, cxn->type);
+
 	switch (cxn->type) {
-	case XST_CONNECTION_ETH:
-		cxn->dev = g_strdup ("eth0");
-		break;
-	case XST_CONNECTION_WVLAN:
-		cxn->dev = g_strdup ("wvlan0");
-		break;
 	case XST_CONNECTION_PPP:
 		cxn->user = TRUE;
 		cxn->autoboot = FALSE;
-		cxn->dev = connection_dev_get_next (root, "ppp");
-		break;
-	case XST_CONNECTION_LO:
-		cxn->dev = g_strdup ("lo");
 		break;
 	case XST_CONNECTION_PLIP:
 		cxn->autoboot = FALSE;
-		cxn->dev = g_strdup ("plip0");
 		break;
+	case XST_CONNECTION_ETH:
+	case XST_CONNECTION_WVLAN:
+	case XST_CONNECTION_IRLAN:
+	case XST_CONNECTION_LO:
 	default:
-		cxn->dev = g_strdup ("NIL");
 		break;
 	}	
 
@@ -692,6 +1042,10 @@ connection_new_from_node (xmlNode *node)
 	}
 
 	cxn->node = node;
+
+	s = xst_xml_get_child_content (node, "file");
+	if (s)
+		cxn->file = s;
 	
 	s = xst_xml_get_child_content (node, "bootproto");
 	if (s) {
@@ -799,6 +1153,99 @@ on_status_enabled_toggled (GtkWidget *w, XstConnection *cxn)
 }
 #endif
 
+static void
+connection_check_netmask_gui (XstConnection *cxn)
+{
+        GtkWidget *netmask_widget, *address_widget;
+        gchar *address, *netmask;
+        guint32 ip1;
+
+	netmask_widget = W ("ip_netmask");
+	address_widget = W ("ip_address");
+        address = gtk_editable_get_chars (GTK_EDITABLE (address_widget), 0, -1);
+        netmask = gtk_editable_get_chars (GTK_EDITABLE (netmask_widget), 0, -1);
+
+        g_print ("ip: %s\n", address);
+        g_print ("netmask: %s\n", netmask);
+
+        if ((sscanf (address, "%d.", &ip1) == 1) && (!strlen (netmask))) {
+		if (ip1 < 127) {
+			gtk_entry_set_text (GTK_ENTRY (netmask_widget), "255.0.0.0");
+		} else if (ip1 < 192) {
+			gtk_entry_set_text (GTK_ENTRY (netmask_widget), "255.255.0.0");
+		} else {
+			gtk_entry_set_text (GTK_ENTRY (netmask_widget), "255.255.255.0");
+		}
+	}
+}
+
+static gchar *
+connection_str_to_addr (gchar *str)
+{
+	gchar **strv;
+	gchar *addr;
+	gint i;
+
+	strv = g_strsplit (str, ".", 4);
+	addr = g_new0 (gchar, 4);
+	
+	for (i = 0; strv[i]; i++)
+		addr[i] = (gchar) atoi (strv[i]);
+
+	g_strfreev (strv);
+	
+	return addr;
+}
+
+static gchar *
+connection_addr_to_str (gchar *addr)
+{
+	gchar *str;
+
+	str = g_strdup_printf ("%u.%u.%u.%u",
+			       (guchar) addr[0],
+			       (guchar) addr[1],
+			       (guchar) addr[2],
+			       (guchar) addr[3]);
+	return str;
+}
+
+static void
+connection_set_bcast_and_network (XstConnection *cxn)
+{
+	gchar *address, *netmask;
+	gchar *broadcast;
+	gchar *network;
+	gint i;
+
+	if (!cxn->address || !*cxn->address ||
+	    !cxn->netmask || !*cxn->netmask)
+		return;
+	
+	address = connection_str_to_addr (cxn->address);
+	netmask = connection_str_to_addr (cxn->netmask);
+	broadcast = g_new0 (gchar, 4);
+	network   = g_new0 (gchar, 4);
+
+	for (i = 0; i < 4; i++) {
+		broadcast[i] = (gchar) (address[i] | (~netmask[i]));
+		network[i]   = (gchar) (address[i] & netmask[i]);
+	}
+
+	if (cxn->broadcast)
+		g_free (cxn->broadcast);
+	if (cxn->network)
+		g_free (cxn->network);
+
+	cxn->broadcast = connection_addr_to_str (broadcast);
+	cxn->network   = connection_addr_to_str (network);
+
+	g_free (address);
+	g_free (netmask);
+	g_free (broadcast);
+	g_free (network);
+}
+
 static void
 empty_general (XstConnection *cxn)
 {
@@ -811,13 +1258,13 @@ empty_general (XstConnection *cxn)
 static void
 empty_ip (XstConnection *cxn)
 {
+	connection_check_netmask_gui (cxn);
+	
 	cxn->ip_config = cxn->tmp_ip_config;
 	GET_BOOL ("ip_", update_dns);
 	GET_STR ("ip_", address);
 	GET_STR ("ip_", netmask);
 	GET_STR ("ip_", gateway);
-
-#warning FIXME: calculate broadcast and stuff in empty_id
 }
 
 static void
@@ -874,6 +1321,7 @@ connection_validate (XstConnection *cxn)
 	switch (cxn->type) {
 	case XST_CONNECTION_ETH:
 	case XST_CONNECTION_WVLAN:
+	case XST_CONNECTION_IRLAN:
 	case XST_CONNECTION_UNKNOWN:
 		if (cxn->ip_config == IP_MANUAL &&
 		    (strempty (cxn->address) ||
@@ -933,6 +1381,8 @@ connection_empty_gui (XstConnection *cxn)
 	case XST_CONNECTION_PLIP:
 		empty_ptp (cxn);
 		break;
+	case XST_CONNECTION_ETH:
+	case XST_CONNECTION_IRLAN:
 	default:
 		empty_ip (cxn);
 		break;
@@ -969,11 +1419,12 @@ connection_config_save (XstConnection *cxn)
 static void
 on_connection_ok_clicked (GtkWidget *w, XstConnection *cxn)
 {
-	if (connection_config_save (cxn))
-		gtk_widget_destroy (cxn->window);
-		
-	if (cxn->modified)
+	if (cxn->modified) {
+		if (connection_config_save (cxn))
+			gtk_widget_destroy (cxn->window);
 		cxn->creating = FALSE;
+	} else
+		gtk_widget_destroy (cxn->window);
 }
 
 static void
@@ -1018,7 +1469,7 @@ on_connection_modified (GtkWidget *w, XstConnection *cxn)
 static void
 on_wvlan_adhoc_toggled (GtkWidget *w, XstConnection *cxn)
 {
-#warning FIXME: implement on_wvlan_adhoc_toggled
+/* FIXME: implement on_wvlan_adhoc_toggled*/
 }
 
 static void
@@ -1029,11 +1480,19 @@ on_ppp_update_dns_toggled (GtkWidget *w, XstConnection *cxn)
 	active = GTK_TOGGLE_BUTTON (W ("ppp_update_dns"))->active;
 
 	gtk_widget_set_sensitive (W ("ppp_dns1_label"), active);
-	gtk_widget_set_sensitive (W ("ppp_dns2_label"), active);
+	gtk_widget_set_sensitive (W ("ppp_dns2_label"), active);	
 	gtk_widget_set_sensitive (W ("ppp_dns1"), active);
 	gtk_widget_set_sensitive (W ("ppp_dns2"), active);
 }
 
+static gboolean
+on_ip_address_focus_out (GtkWidget *widget, GdkEventFocus *event, XstConnection *cxn)
+{
+	connection_check_netmask_gui (cxn);
+
+        return TRUE;
+}
+
 static void
 fill_general (XstConnection *cxn)
 {
@@ -1089,7 +1548,7 @@ fill_ip (XstConnection *cxn)
 	GtkWidget *menu, *menuitem, *omenu;
 	IPConfigType i;
 
-	char *blah[] = {
+	char *bootproto_labels[] = {
 		N_("Manual"),
 		N_("DHCP"),
 		N_("BOOTP")
@@ -1099,7 +1558,7 @@ fill_ip (XstConnection *cxn)
 
 	menu = gtk_menu_new ();
 	for (i = 0; i < 3; i++) {
-		menuitem = gtk_menu_item_new_with_label (_(blah[i]));
+		menuitem = gtk_menu_item_new_with_label (_(bootproto_labels[i]));
 		gtk_object_set_user_data (GTK_OBJECT (menuitem), cxn);
 		gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
 				    GTK_SIGNAL_FUNC (ip_config_menu_cb),
@@ -1168,7 +1627,8 @@ static void
 hookup_callbacks (XstConnection *cxn)
 {
 	gint i;
-	WidgetSignal signals[] = {
+	WidgetSignal signals[] =
+	{
 		{ "on_connection_ok_clicked", on_connection_ok_clicked },
 		{ "on_connection_cancel_clicked", on_connection_cancel_clicked },
 		{ "on_connection_config_dialog_delete_event", on_connection_config_dialog_delete_event },
@@ -1177,12 +1637,20 @@ hookup_callbacks (XstConnection *cxn)
 		{ "on_connection_config_dialog_destroy", on_connection_config_dialog_destroy },
 		{ "on_wvlan_adhoc_toggled", on_wvlan_adhoc_toggled },
 		{ "on_ppp_update_dns_toggled", on_ppp_update_dns_toggled },
+		{ "on_ip_address_focus_out", on_ip_address_focus_out },
+		{ "on_ip_netmask_focus_out", on_ip_address_focus_out },
 		{ NULL }
 	};
-	struct { char *name; EditableFilterRules rule; }
-	s[] = {
+
+	struct
+	{
+		char *name;
+		EditableFilterRules rule;
+	}
+	s [] =
+	{
 		{ "ip_address",          EF_ALLOW_IP },
-		{ "ip_netmask",          EF_ALLOW_IP },
+      		{ "ip_netmask",          EF_ALLOW_IP },
 		{ "ip_gateway",          EF_ALLOW_IP },
 		{ "ppp_dns1",            EF_ALLOW_IP },
 		{ "ppp_dns2",            EF_ALLOW_IP },
@@ -1202,8 +1670,7 @@ void
 connection_configure (XstConnection *cxn)
 {
 	GtkWidget *nb;
-/*	GtkWidget *hb, *qm;*/
-	gchar *s, *smart_dhcpcd;
+	gchar *s;
 
 	if (cxn->window) {
 		gtk_widget_show (cxn->window);
@@ -1226,17 +1693,15 @@ connection_configure (XstConnection *cxn)
 	fill_general (cxn);
 	fill_ip      (cxn);
 
-	smart_dhcpcd = xst_xml_get_child_content (cxn->node->parent, "smartdhcpcd");
-	if (!smart_dhcpcd || !atoi (smart_dhcpcd)) {
-		GtkWidget *w = W("ip_update_dns");
-		gtk_widget_hide (w);
-	}
-
-	g_free (smart_dhcpcd);
+	if (!xst_xml_element_get_boolean (cxn->node->parent, "smartdhcpcd"))
+		gtk_widget_hide (W("ip_update_dns"));
+	if (!xst_xml_element_get_boolean (cxn->node->parent, "userifacfectl"))
+		gtk_widget_hide (W("status_user"));
 
 	/* would like to do this as a switch */
 	nb = W ("connection_nb");
 	if (cxn->type == XST_CONNECTION_PPP) {
+		xst_ui_image_set_pix (W ("connection_pixmap"), PIXMAPS_DIR "/ppp.png");
 		fill_ppp (cxn);
 		fill_ppp_adv (cxn);
 		gtk_notebook_remove_page (GTK_NOTEBOOK (nb),
@@ -1251,14 +1716,20 @@ connection_configure (XstConnection *cxn)
 								 W ("ppp_adv_vbox")));
 	}
        
-	if (cxn->type == XST_CONNECTION_WVLAN)
+	if (cxn->type == XST_CONNECTION_WVLAN) {
+		xst_ui_image_set_pix (W ("connection_pixmap"), PIXMAPS_DIR "/wavelan-48.png");
 		fill_wvlan (cxn);
-	else
+		/* FIXME: temprorarily disabling this notebook, until we get support for this. */
+		gtk_notebook_remove_page (GTK_NOTEBOOK (nb),
+					  gtk_notebook_page_num (GTK_NOTEBOOK (nb),
+								 W ("wvlan_vbox")));
+	} else
 		gtk_notebook_remove_page (GTK_NOTEBOOK (nb),
 					  gtk_notebook_page_num (GTK_NOTEBOOK (nb),
 								 W ("wvlan_vbox")));
 
 	if (cxn->type == XST_CONNECTION_PLIP) {
+		xst_ui_image_set_pix (W ("connection_pixmap"), PIXMAPS_DIR "/plip-48.png");
 		fill_ptp (cxn);
 		gtk_notebook_remove_page (GTK_NOTEBOOK (nb),
 					  gtk_notebook_page_num (GTK_NOTEBOOK (nb),
@@ -1268,6 +1739,14 @@ connection_configure (XstConnection *cxn)
 					  gtk_notebook_page_num (GTK_NOTEBOOK (nb),
 								 W ("ptp_vbox")));
 
+	if (cxn->type == XST_CONNECTION_ETH) {
+		xst_ui_image_set_pix (W ("connection_pixmap"), PIXMAPS_DIR "/connection-ethernet.png");
+	}
+
+	if (cxn->type == XST_CONNECTION_IRLAN) {
+		xst_ui_image_set_pix (W ("connection_pixmap"), PIXMAPS_DIR "/irda-48.png");
+	}
+
 	cxn->frozen = FALSE;
 
 	connection_set_modified (cxn, FALSE);
@@ -1275,6 +1754,27 @@ connection_configure (XstConnection *cxn)
 	gtk_widget_show (cxn->window);
 }
 
+static gboolean
+connection_updatedns_supported (XstConnection *cxn)
+{
+	switch (cxn->type) {
+	case XST_CONNECTION_ETH:
+	case XST_CONNECTION_WVLAN:
+	case XST_CONNECTION_PPP:
+	case XST_CONNECTION_IRLAN:
+		return TRUE;
+	case XST_CONNECTION_PLIP:
+	case XST_CONNECTION_OTHER:
+	case XST_CONNECTION_LO:
+	case XST_CONNECTION_UNKNOWN:
+	case XST_CONNECTION_LAST:
+	default:
+		return FALSE;
+	}
+
+	return FALSE;
+}
+
 void
 connection_save_to_node (XstConnection *cxn, xmlNode *root)
 {
@@ -1293,9 +1793,11 @@ connection_save_to_node (XstConnection *cxn, xmlNode *root)
 	connection_xml_save_boolean_to_node (node, "user", cxn->user);
 	connection_xml_save_boolean_to_node (node, "auto", cxn->autoboot);
 	connection_xml_save_boolean_to_node (node, "enabled", cxn->enabled);
-	connection_xml_save_boolean_to_node (node, "update_dns", cxn->update_dns);
+	if (connection_updatedns_supported (cxn))
+		connection_xml_save_boolean_to_node (node, "update_dns", cxn->update_dns);
 
 	/* TCP/IP general paramaters */
+	connection_set_bcast_and_network (cxn);
 	connection_xml_save_str_to_node (node, "address", cxn->address);
 	connection_xml_save_str_to_node (node, "netmask", cxn->netmask);
 	connection_xml_save_str_to_node (node, "broadcast", cxn->broadcast);
