@@ -56,7 +56,7 @@ static char *ntp_servers[] =
 	"ntp1.cmc.ec.gc.ca (Eastern Canada)",
 	"ntp2.cmc.ec.gc.ca (Eastern Canada)",
 	"clock.tricity.wsu.edu (Washington, USA)",
-	"wuarchive.wustl.edu (Washington, USA)",
+	"wuarchive.wustl.edu (Missouri, USA)",
 	"clock.psu.edu (Pennsylvania, USA)",
 	"constellation.ecn.uoknor.edu (Oklahoma, USA)",
 	"gilbreth.ecn.purdue.edu (Indiana, USA)",
@@ -85,6 +85,32 @@ static char *ntp_servers[] =
 	"ntp.adelaide.edu.au (South Australia)",
 	"ntp.shim.org (Singapore, Asia)",
 	"time.nuri.net (Korea, Asia)",
+	"ntp.cs.mu.oz.au (Melbourne, Austrilia)",
+	"ntp.mel.nml.csiro.au (Melbourne, Australia)",
+	"ntp.nml.csiro.au (Sydney, Australia)",
+	"ntp.per.nml.csiro.au (Perth, Australia)",
+	"swisstime.ethz.ch (Zurich, Switzerland)",
+	"ntp.cesnet.cz (Prague, Czech Republic)",
+	"ntpa2.kph.uni-mainz.de (Mainz, Germany)",
+	"ntps1-0.cs.tu-berlin.de (Berlin, Germany)",
+	"ntps1-1.cs.tu-berlin.de (Berlin, Germany)",
+	"ntps1-2.uni-erlangen.de (Erlangen, Germany)",
+	"canon.inria.fr (Rocquencourt, France)",
+	"chronos.cru.fr (Britany, France)",
+	"stdtime.gov.hk (Hong Kong, China)",
+	"clock.cuhk.edu.hk (Hong Kong, China)",
+	"time.ien.it (Torino, Italy)",
+	"clock.tl.fukuoka-u.ac.jp (Fukuoka, Japan)",
+	"cronos.cenam.mx (Queretaro, Mexico)",
+	"ntp0.nl.net (Amsterdam, The Netherlands)",
+	"ntp1.nl.net (Amsterdam, The Netherlands)",
+	"ntp2.nl.net (Amsterdam, The Netherlands)",
+	"time.service.uit.no (Norway)",
+	"ntp.certum.pl (Poland)",
+	"vega.cbk.poznan.pl (Borowiec, Poland)",
+	"time1.stupi.se (Stockholm, Sweden)",
+	"goodtime.ijs.si (Ljubljana, Slovenia)",
+	"ntp2.ja.net (UK)",
 	0
 };
 
