@@ -21,7 +21,12 @@
  * Authors: Tambet Ingo <tambet@ximian.com>.
  */
 
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
 #include <stdlib.h>
+#include <ctype.h>
 #include <gnome.h>
 
 #include "xst.h"
@@ -157,6 +162,40 @@ user_account_passwd_changed (GtkEditable *entry, gpointer data)
 	gtk_object_set_data (GTK_OBJECT (entry), USER_ACCOUNT_PASSWD_CHANGED, GINT_TO_POINTER (TRUE));
 }
 
+static void
+user_account_comment_changed (GtkEditable *editable, gchar *new_text, gint new_text_length,
+			      gint *position, gpointer data)
+{
+	gint i;
+	gchar *buf = NULL;
+	UserAccountGui *gui = data;
+	
+	gtk_signal_handler_block_by_func (GTK_OBJECT (editable),
+					  GTK_SIGNAL_FUNC (user_account_comment_changed),
+					  data);
+
+	for (i = 0; i < new_text_length; i++) {
+		if (iscntrl (new_text[i]) || new_text[i] == ',' ||
+		    new_text[i] == '=' || new_text[i] == ':') {
+			buf = g_strdup_printf (N_("Invalid character '%c'.\n"
+				"This field doesn't accept control characters ',' ':' and '='."),
+					       new_text[i]);
+			break;
+		}
+	}
+
+	if (buf)
+		user_account_gui_error (GTK_WINDOW (gui->top), buf);
+	else
+		gtk_editable_insert_text (editable, new_text, new_text_length, position);	
+
+	gtk_signal_handler_unblock_by_func (GTK_OBJECT (editable),
+					    GTK_SIGNAL_FUNC (user_account_comment_changed),
+					    data);
+	
+	gtk_signal_emit_stop_by_name (GTK_OBJECT (editable), "insert_text");
+}
+
 static void
 user_account_grab_focus (GtkWidget *w, gpointer data)
 {
@@ -187,13 +226,82 @@ get_group_list (gchar *field, xmlNodePtr user_node)
 	     u = xst_xml_element_find_next (u, "group"))
 	{
 
-		if (check_node_complexity (u))
+		if (check_node_visibility (u))
 			list = g_list_prepend (list, xst_xml_get_child_content (u, field));
 	}
 
 	return list;
 }
 
+static void
+user_account_set_modified (GtkWidget *w, gpointer data)
+{
+	UserAccountGui *gui;
+
+	gui = (UserAccountGui *) data;
+	if (gui->allow_modify)
+		gui->account->modified = TRUE;
+}
+
+static void
+user_account_connect_signals (UserAccountGui *gui)
+{
+	struct {
+		void *obj;
+		gchar *signal;
+		void *func;
+		void *data;
+		gboolean modifies;
+	} table[] = {
+		{ gui->profile_button, "clicked",      profile_table_run,              NULL, FALSE },
+		{ gui->add,            "clicked",      user_account_gui_add,           gui,  TRUE },
+		{ gui->remove,         "clicked",      user_account_gui_remove,        gui,  TRUE },
+		{ gui->all,            "select_row",   user_account_gui_all_select,    gui,  FALSE },
+		{ gui->all,            "unselect_row", user_account_gui_all_select,    gui,  FALSE },
+		{ gui->member,         "select_row",   user_account_gui_member_select, gui,  FALSE },
+		{ gui->member,         "unselect_row", user_account_gui_member_select, gui,  FALSE },
+		{ gui->pwd_manual,     "toggled",      user_account_passwd_toggled,    gui,  TRUE },
+		{ gui->pwd_random,     "toggled",      user_account_passwd_toggled,    gui,  TRUE },
+		{ gui->pwd_random_new, "clicked",      user_account_passwd_random_new, gui,  TRUE },
+		{ gui->pwd1,           "changed",      user_account_passwd_changed,    gui,  TRUE },
+		{ gui->pwd2,           "changed",      user_account_passwd_changed,    gui,  TRUE },
+		{ gui->name,           "activate",     user_account_grab_focus,        gui->comment, TRUE },
+		{ gui->comment,        "activate",     user_account_grab_focus,        gui->office, TRUE },
+		{ gui->office,         "activate",     user_account_grab_focus,        gui->wphone, TRUE },
+		{ gui->wphone,         "activate",     user_account_grab_focus,        gui->hphone, TRUE },
+		{ gui->pwd1,           "activate",     user_account_grab_focus,        gui->pwd2, FALSE },
+		{ gui->uid,            "activate",     user_account_grab_focus,        gui->home, TRUE },
+		{ gui->comment,        "insert-text",  user_account_comment_changed,   gui,  TRUE },
+		{ gui->office,         "insert-text",  user_account_comment_changed,   gui,  TRUE },
+		{ gui->wphone,         "insert-text",  user_account_comment_changed,   gui,  TRUE },
+		{ gui->hphone,         "insert-text",  user_account_comment_changed,   gui,  TRUE },
+
+		{ gui->group_entry,    NULL,           NULL,                           NULL, TRUE },
+		{ gui->min,            NULL,           NULL,                           NULL, TRUE },
+		{ gui->max,            NULL,           NULL,                           NULL, TRUE },
+		{ gui->days,           NULL,           NULL,                           NULL, TRUE },
+		{ gui->profile_menu,   NULL,           NULL,                           NULL, TRUE },
+		{ gui->shell_entry,    NULL,           NULL,                           NULL, TRUE },
+		{ gui->home,           NULL,           NULL,                           NULL, TRUE },
+		{ NULL, NULL, NULL, NULL, FALSE }
+	};
+	gint i;
+
+	for (i = 0; table[i].obj; i++) {
+		if (table[i].signal)
+			gtk_signal_connect (GTK_OBJECT (table[i].obj), table[i].signal,
+					    GTK_SIGNAL_FUNC (table[i].func), table[i].data);
+		if (table[i].modifies) {
+			if (gtk_signal_lookup ("changed", GTK_OBJECT_TYPE (GTK_OBJECT (table[i].obj))))
+				gtk_signal_connect (GTK_OBJECT (table[i].obj), "changed",
+						    user_account_set_modified, gui);
+			else if (gtk_signal_lookup ("clicked", GTK_OBJECT_TYPE (GTK_OBJECT (table[i].obj))))
+				gtk_signal_connect (GTK_OBJECT (table[i].obj), "clicked",
+						    user_account_set_modified, gui);
+		}
+	}
+}
+
 UserAccountGui *
 user_account_gui_new (UserAccount *account, GtkWidget *parent)
 {
@@ -206,6 +314,7 @@ user_account_gui_new (UserAccount *account, GtkWidget *parent)
 	gui->account = account;
 	gui->xml = glade_xml_new (tool->glade_path, NULL);
 	gui->top = parent;
+	gui->allow_modify = TRUE;
 
 	gui->basic_frame = glade_xml_get_widget (gui->xml, "user_settings_basic");
 	gui->name    = GTK_ENTRY (glade_xml_get_widget (gui->xml, "user_settings_name"));
@@ -216,6 +325,7 @@ user_account_gui_new (UserAccount *account, GtkWidget *parent)
 	gui->hphone  = GTK_ENTRY (glade_xml_get_widget (gui->xml, "user_settings_hphone"));
 	gui->home    = GTK_ENTRY (glade_xml_get_widget (gui->xml, "user_settings_home"));
 	gui->shell   = GTK_COMBO (glade_xml_get_widget (gui->xml, "user_settings_shell"));
+	gui->shell_entry = GTK_ENTRY (glade_xml_get_widget (gui->xml, "user_settings_shell_entry"));
 	gui->uid     = GTK_SPIN_BUTTON (glade_xml_get_widget (gui->xml, "user_settings_uid"));
 	gui->advanced = glade_xml_get_widget (gui->xml, "user_settings_advanced");
 	gui->profile_box = glade_xml_get_widget (gui->xml, "user_settings_profile_box");
@@ -225,6 +335,7 @@ user_account_gui_new (UserAccount *account, GtkWidget *parent)
 	gui->group_box = glade_xml_get_widget (gui->xml, "user_settings_group_box");
 	gui->group_extra = glade_xml_get_widget (gui->xml, "user_settings_group_extra");
 	gui->group   = GTK_COMBO (glade_xml_get_widget (gui->xml, "user_settings_group"));
+	gui->group_entry = GTK_ENTRY (glade_xml_get_widget (gui->xml, "user_settings_group_entry"));
 	gui->all     = GTK_CLIST (glade_xml_get_widget (gui->xml, "user_settings_gall"));
 	gui->member  = GTK_CLIST (glade_xml_get_widget (gui->xml, "user_settings_gmember"));
 	gui->add     = glade_xml_get_widget (gui->xml, "user_settings_add");
@@ -245,49 +356,8 @@ user_account_gui_new (UserAccount *account, GtkWidget *parent)
 	gui->max  = GTK_SPIN_BUTTON (glade_xml_get_widget (gui->xml, "user_passwd_max"));
 	gui->days = GTK_SPIN_BUTTON (glade_xml_get_widget (gui->xml, "user_passwd_days"));
 
-	gtk_signal_connect (GTK_OBJECT (gui->profile_button), "clicked",
-			    GTK_SIGNAL_FUNC (profile_table_run), NULL);
+	user_account_connect_signals (gui);
 	
-	gtk_signal_connect (GTK_OBJECT (gui->add), "clicked",
-			    GTK_SIGNAL_FUNC (user_account_gui_add), gui);
-	gtk_signal_connect (GTK_OBJECT (gui->remove), "clicked",
-			    GTK_SIGNAL_FUNC (user_account_gui_remove), gui);
-
-	gtk_signal_connect (GTK_OBJECT (gui->all), "select_row",
-			    GTK_SIGNAL_FUNC (user_account_gui_all_select), gui);
-	gtk_signal_connect (GTK_OBJECT (gui->all), "unselect_row",
-			    GTK_SIGNAL_FUNC (user_account_gui_all_select), gui);
-
-	gtk_signal_connect (GTK_OBJECT (gui->member), "select_row",
-			    GTK_SIGNAL_FUNC (user_account_gui_member_select), gui);
-	gtk_signal_connect (GTK_OBJECT (gui->member), "unselect_row",
-			    GTK_SIGNAL_FUNC (user_account_gui_member_select), gui);
-
-	gtk_signal_connect (GTK_OBJECT (gui->pwd_manual), "toggled",
-			    GTK_SIGNAL_FUNC (user_account_passwd_toggled), gui);
-	gtk_signal_connect (GTK_OBJECT (gui->pwd_random), "toggled",
-			    GTK_SIGNAL_FUNC (user_account_passwd_toggled), gui);
-	gtk_signal_connect (GTK_OBJECT (gui->pwd_random_new), "clicked",
-			    GTK_SIGNAL_FUNC (user_account_passwd_random_new), gui);
-
-	gtk_signal_connect (GTK_OBJECT (gui->pwd1), "changed",
-			    GTK_SIGNAL_FUNC (user_account_passwd_changed), gui);
-	gtk_signal_connect (GTK_OBJECT (gui->pwd2), "changed",
-			    GTK_SIGNAL_FUNC (user_account_passwd_changed), gui);
-
-	gtk_signal_connect (GTK_OBJECT (gui->name), "activate",
-			    GTK_SIGNAL_FUNC (user_account_grab_focus), (gpointer) gui->comment);
-	gtk_signal_connect (GTK_OBJECT (gui->comment), "activate",
-			    GTK_SIGNAL_FUNC (user_account_grab_focus), (gpointer) gui->office);
-	gtk_signal_connect (GTK_OBJECT (gui->office), "activate",
-			    GTK_SIGNAL_FUNC (user_account_grab_focus), (gpointer) gui->wphone);
-	gtk_signal_connect (GTK_OBJECT (gui->wphone), "activate",
-			    GTK_SIGNAL_FUNC (user_account_grab_focus), (gpointer) gui->hphone);
-	gtk_signal_connect (GTK_OBJECT (gui->pwd1), "activate",
-			    GTK_SIGNAL_FUNC (user_account_grab_focus), (gpointer) gui->pwd2);
-	gtk_signal_connect (GTK_OBJECT (gui->uid), "activate",
-			    GTK_SIGNAL_FUNC (user_account_grab_focus), (gpointer) gui->home);
-
 	return gui;
 }
 
@@ -421,6 +491,9 @@ setup_advanced_add (UserAccountGui *gui, GtkWidget *notebook)
 	}
 	g_slist_free (list);
 	xst_ui_option_menu_set_selected_string (gui->profile_menu, pf->name);
+
+	/* FIXME: Temporarily hiding all profile-related stuff. */
+	gtk_widget_hide (GTK_WIDGET (gui->profile_box));
 }
 
 static void
@@ -493,6 +566,8 @@ user_account_gui_setup (UserAccountGui *gui, GtkWidget *top)
 	GtkWidget *notebook;
 	GList *users, *items;
 	UserAccount *account = gui->account;
+
+	gui->allow_modify = FALSE;
 	
 	user_account_shells_setup (gui);
 	user_account_groups_setup (gui->group, account->node);
@@ -555,6 +630,8 @@ user_account_gui_setup (UserAccountGui *gui, GtkWidget *top)
 		gtk_widget_show (notebook);
 		gtk_container_add (GTK_CONTAINER (top), notebook);
 	}
+
+	gui->allow_modify = TRUE;
 }
 
 static gchar**
@@ -576,12 +653,70 @@ user_account_gui_save_comment (UserAccountGui *gui)
 	return val;
 }
 
+static gchar *
+warnings_join (GList *warnings)
+{
+	GList *l;
+	gchar *text = NULL;
+	gchar *text2;
+
+	for (l = warnings; l; l = l->next) {
+		text2 = text;
+		if (text)
+			text = g_strdup_printf ("%s\n%s", text, (gchar *) l->data);
+		else
+			text = g_strdup ((gchar *) l->data);
+		g_free (text2);
+	}
+
+	text2 = text;
+	text = g_strdup_printf ("The information you just entered produced the following warnings:\n\n%s\n\nContinue anyways?", text);
+	g_free (text2);
+
+	return text;
+}
+
+static void
+gui_warnings_cb (gint button, gpointer data)
+{
+	gboolean *reply = (gboolean *) data;
+
+	if (button == 0) /* Yes button */
+		*reply = TRUE;
+	else /* No button */
+		*reply = FALSE;
+}
+
+static gboolean
+user_account_gui_warnings (GtkWindow *parent, GList *warnings)
+{
+	GtkWidget *d;
+	gchar *text;
+	gboolean reply;
+	
+	g_return_val_if_fail (warnings != NULL, TRUE);
+
+	text = warnings_join (warnings);
+	g_return_val_if_fail (text != NULL, TRUE);
+
+	if (parent)
+		d = gnome_question_dialog_parented (text, gui_warnings_cb, &reply, parent);
+	else
+		d = gnome_question_dialog (text, gui_warnings_cb, &reply);
+
+	gnome_dialog_run (GNOME_DIALOG (d));
+	g_free (text);
+
+	return reply;
+}
+
 gboolean
 user_account_gui_save (UserAccountGui *gui)
 {
 	UserAccount *account;
 	xmlNodePtr node = gui->account->node;
 	GtkWindow *parent = GTK_WINDOW (gui->top);
+	GList *warnings;
 	gchar *buf, *error;
 	gint row = 0;
 
@@ -641,11 +776,18 @@ user_account_gui_save (UserAccountGui *gui)
 		user_account_gui_error (parent, error);
 		user_account_destroy (account);
 		return FALSE;
-	} else {
-		user_account_destroy (gui->account);
-		gui->account = account;
-		return TRUE;
+	} else if ((warnings = user_account_check_warnings (account))) {
+		if (!user_account_gui_warnings (parent, warnings)) {
+			user_account_destroy_warnings (warnings);
+			user_account_destroy (account);
+			return FALSE;
+		}
+		user_account_destroy_warnings (warnings);
 	}
+
+	user_account_destroy (gui->account);
+	gui->account = account;
+	return TRUE;
 }
 
 void
